@@ -1,4 +1,4 @@
-// Copyright © 2011, 2012, 2014-17 Richard Kettlewell.
+// Copyright © 2011, 2012, 2014-17, 2019 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,10 +24,11 @@
 #include "Command.h"
 #include "Utils.h"
 #include "Database.h"
-#include "Prune.h"
+#include "PrunePolicy.h"
 #include "ConfDirective.h"
 #include "Device.h"
 #include "Indent.h"
+#include "BackupPolicy.h"
 #include <cerrno>
 #include <regex>
 #include <sstream>
@@ -78,6 +79,12 @@ void Conf::write(std::ostream &os, int step, bool verbose) const {
   os << indent(step) << "logs " << quote(logs) << '\n';
   d(os, "", step);
 
+  d(os, "# Path to database", step);
+  d(os, "#  database PATH", step);
+  if(database.size())
+    os << indent(step) << "database " << quote(database) << '\n';
+  d(os, "", step);
+
   d(os, "# Path to lock file", step);
   d(os, "#  lock PATH", step);
   if(lock.size())
@@ -112,13 +119,6 @@ void Conf::write(std::ostream &os, int step, bool verbose) const {
      << indent(step) << "color-bad 0x" << colorBad << '\n';
   d(os, "", step);
 
-  if(reportPruneLogs) {
-    d(os, "# How many days worth of pruning logs to report", step);
-    d(os, "#  report-prune-logs DAYS", step);
-    os << indent(step) << "report-prune-logs " << reportPruneLogs << '\n';
-    d(os, "", step);
-  }
-
   d(os, "# Path to mail transport agent", step);
   d(os, "#  sendmail PATH", step);
   os << indent(step) << "sendmail " << quote(sendmail) << '\n';
@@ -254,7 +254,7 @@ void Conf::write(std::ostream &os, int step, bool verbose) const {
 
 // Read the master configuration file plus anything it includes.
 void Conf::read() {
-  readOneFile(configPath);
+  readOneFile(globalConfigPath);
 }
 
 // Read one configuration file.  Throws IOError if some file cannot be
@@ -340,8 +340,10 @@ void Conf::includeFile(const std::string &path) {
 
 void Conf::validate() const {
   for(auto &h: hosts)
-    for(auto &v: h.second->volumes)
+    for(auto &v: h.second->volumes) {
+      validateBackupPolicy(v.second);
       validatePrunePolicy(v.second);
+    }
 }
 
 // (De-)select all hosts
@@ -410,7 +412,7 @@ void Conf::readState() {
     return;
   std::string hostName, volumeName;
   std::vector<std::string> files;
-  const bool progress = (warning_mask & WARNING_VERBOSE) && isatty(2);
+  const bool progress = (globalWarningMask & WARNING_VERBOSE) && isatty(2);
   std::vector<std::string> upgraded;
 
   std::string log;
@@ -428,7 +430,6 @@ void Conf::readState() {
       backup.deviceName = stmt.get_string(2);
       backup.id = stmt.get_string(3);
       backup.time = stmt.get_int64(4);
-      backup.date = Date(backup.time);
       backup.pruned = stmt.get_int64(5);
       backup.rc = stmt.get_int(6);
       backup.setStatus(stmt.get_int(7));
@@ -449,9 +450,8 @@ void Conf::readState() {
       if(!std::regex_match(files[n], mr, logfileRegexp))
         continue;
       Backup backup;
-      backup.date = Date(mr[1]);
       backup.id = mr[1];
-      backup.time = backup.date.toTime();
+      backup.time = Date(mr[1]).toTime();
       backup.deviceName = mr[2];
       hostName = mr[3];
       volumeName = mr[4];
@@ -491,7 +491,7 @@ void Conf::readState() {
 
       addBackup(backup, hostName, volumeName, true);
 
-      if(command.act) {
+      if(globalCommand.act) {
         // addBackup might fail to set volume
         if(backup.volume != nullptr) {
           if(upgraded.size() == 0)
@@ -511,7 +511,7 @@ void Conf::readState() {
       }
     }
     logsRead = true;
-    if(command.act && upgraded.size()) {
+    if(globalCommand.act && upgraded.size()) {
       getdb().commit();
       bool upgradeFailure = false;
       for(std::string &u: upgraded) {
@@ -533,7 +533,7 @@ void Conf::addBackup(Backup &backup,
                      const std::string &hostName,
                      const std::string &volumeName,
                      bool forceWarn) {
-  const bool progress = (warning_mask & WARNING_VERBOSE) && isatty(2);
+  const bool progress = (globalWarningMask & WARNING_VERBOSE) && isatty(2);
   unsigned warning_type = forceWarn ? WARNING_ALWAYS : WARNING_UNKNOWN;
 
   /* Don't keep pruned backups around */
@@ -547,7 +547,7 @@ void Conf::addBackup(Backup &backup,
       warning(warning_type,
               "unknown device %s", backup.deviceName.c_str());
       unknownDevices.insert(backup.deviceName);
-      ++config.unknownObjects;
+      ++globalConfig.unknownObjects;
     }
     return;
   }
@@ -560,7 +560,7 @@ void Conf::addBackup(Backup &backup,
         progressBar(IO::err, nullptr, 0, 0);
       warning(warning_type, "unknown host %s", hostName.c_str());
       unknownHosts.insert(hostName);
-      ++config.unknownObjects;
+      ++globalConfig.unknownObjects;
     }
     return;
   }
@@ -572,7 +572,7 @@ void Conf::addBackup(Backup &backup,
       warning(warning_type, "unknown volume %s:%s",
               hostName.c_str(), volumeName.c_str());
       host->unknownVolumes.insert(volumeName);
-      ++config.unknownObjects;
+      ++globalConfig.unknownObjects;
     }
     return;
   }
@@ -612,7 +612,7 @@ void Conf::identifyDevices(int states) {
   }
   if(!found && states == Store::Enabled) {
     error("no backup devices found");
-    if(!(warning_mask & WARNING_STORE))
+    if(!(globalWarningMask & WARNING_STORE))
       for(size_t n = 0; n < storeExceptions.size(); ++n)
         IO::err.writef("  %s\n", storeExceptions[n].what());
   }
@@ -621,15 +621,19 @@ void Conf::identifyDevices(int states) {
 
 Database &Conf::getdb() {
   if(!db) {
-    if(database.size() == 0)
-      database = logs + "/backups.db";
-    if(command.act) {
-      db = new Database(database);
+    if(globalDatabase.size() == 0) {
+      if(database.size() != 0)
+        globalDatabase = database;
+      else
+        globalDatabase = logs + "/" DEFAULT_DATABASE;
+    }
+    if(globalCommand.act) {
+      db = new Database(globalDatabase);
       if(!db->hasTable("backup"))
         createTables();
     } else {
       try {
-        db = new Database(database, false);
+        db = new Database(globalDatabase, false);
       } catch(DatabaseError &) {
         db = new Database(":memory:");
         createTables();
@@ -664,4 +668,4 @@ std::string Conf::what() const {
   return "system";
 }
 
-Conf config;
+Conf globalConfig;
