@@ -1,5 +1,5 @@
 // -*-C++-*-
-// Copyright © 2011, 2012, 2014-2016 Richard Kettlewell.
+// Copyright © 2011, 2012, 2014-2016, 2019 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -109,15 +109,15 @@ public:
   /** @brief Log directory */
   std::string logs = DEFAULT_LOGS;
 
+  /** @brief Database path */
+  std::string database;
+
   /** @brief Lockfile path */
   std::string lock;
 
   /** @brief Age to keep pruning logs */
   int keepPruneLogs = DEFAULT_KEEP_PRUNE_LOGS;
 
-  /** @brief Age to report pruning logs */
-  int reportPruneLogs = 0;
-
   /** @brief Path to @c sendmail */
   std::string sendmail = DEFAULT_SENDMAIL;
 
@@ -373,6 +373,6 @@ private:
 };
 
 /** @brief Global configuration */
-extern Conf config;
+extern Conf globalConfig;
 
 #endif /* CONF_H */
