@@ -1,4 +1,4 @@
-// Copyright © 2011, 2012, 2014-17 Richard Kettlewell.
+// Copyright © 2011, 2012, 2014-19 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -139,6 +139,11 @@ void ColorDirective::set_packed(ConfContext &cc, size_t n, int radix) const {
 
 // Global directives ----------------------------------------------------------
 
+/** @brief Parse arguments to the @c store directive
+ * @param cc Configuration context
+ * @param mounted Set to whether the store must be mounted
+ * @return Index of the argument after the options
+ */
 size_t parseStoreArguments(const ConfContext &cc, bool &mounted) {
    mounted = true;
    size_t i = 1;
@@ -187,18 +192,6 @@ static const struct StyleSheetDirective: public ConfDirective {
   }
 } stylesheet_directive;
 
-/** @brief The @c colors directive */
-static const struct ColorsDirective: public ConfDirective {
-  ColorsDirective(): ConfDirective("colors", 2, 2) {}
-  void set(ConfContext &cc) const override {
-    warning(WARNING_DEPRECATED,
-            "%s:%d: the 'colors' directive is deprecated, use 'color-good' and 'color-bad' instead",
-            cc.path.c_str(), cc.line);
-    cc.conf->colorGood = parseInteger(cc.bits[1], 0, 0xFFFFFF, 0);
-    cc.conf->colorBad = parseInteger(cc.bits[2], 0, 0xFFFFFF, 0);
-  }
-} colors_directive;
-
 /** @brief The @c color-good directive */
 static const struct ColorGoodDirective: public ColorDirective {
   ColorGoodDirective(): ColorDirective("color-good") {}
@@ -241,7 +234,7 @@ static const struct MaxFileUsageDirective: public ConfDirective {
 
 /** @brief The @c public directive */
 static const struct PublicDirective: public ConfDirective {
-  PublicDirective(): ConfDirective("public", 0, 1) {}
+  PublicDirective(): ConfDirective("public", 1, 1) {}
   void set(ConfContext &cc) const override {
     cc.conf->publicStores = get_boolean(cc);
   }
@@ -255,6 +248,14 @@ static const struct LogsDirective: public ConfDirective {
   }
 } logs_directive;
 
+/** @brief The @c database directive */
+static const struct DatabaseDirective: public ConfDirective {
+  DatabaseDirective(): ConfDirective("database", 1, 1) {}
+  void set(ConfContext &cc) const override {
+    cc.conf->database = cc.bits[1];
+  }
+} database_directive;
+
 /** @brief The @c lock directive */
 static const struct LockDirective: public ConfDirective {
   LockDirective(): ConfDirective("lock", 1, 1) {}
@@ -291,21 +292,11 @@ static const struct PostAccessHookDirective: public ConfDirective {
 static const struct KeepPruneLogsDirective: public ConfDirective {
   KeepPruneLogsDirective(): ConfDirective("keep-prune-logs", 1, 1) {}
   void set(ConfContext &cc) const override {
-    cc.conf->keepPruneLogs = parseInteger(cc.bits[1], 1);
+    cc.conf->keepPruneLogs = parseInteger(cc.bits[1], 1,
+                                          std::numeric_limits<int>::max());
   }
 } keep_prune_logs_directive;
 
-/** @brief The @c report-prune-logs directive */
-static const struct ReportPruneLogsDirective: public ConfDirective {
-  ReportPruneLogsDirective(): ConfDirective("report-prune-logs", 1, 1) {}
-  void set(ConfContext &cc) const override {
-    warning(WARNING_DEPRECATED,
-            "%s:%d: the 'report-prune-logs' directive is deprecated, use 'report' instead",
-            cc.path.c_str(), cc.line);
-    cc.conf->reportPruneLogs = parseInteger(cc.bits[1], 1);
-  }
-} report_prune_logs_directive;
-
 /** @brief The @c include directive */
 static const struct IncludeDirective: public ConfDirective {
   IncludeDirective(): ConfDirective("include", 1, 1) {}
@@ -483,33 +474,46 @@ static const struct GraphLayoutDirective: public ConfDirective {
 static const struct MaxAgeDirective: InheritableDirective {
   MaxAgeDirective(): InheritableDirective("max-age", 1, 1) {}
   void set(ConfContext &cc) const override {
-    cc.context->maxAge = parseInteger(cc.bits[1], 1);
+    cc.context->maxAge = parseInteger(cc.bits[1], 1,
+                                      std::numeric_limits<int>::max());
   }
 } max_age_directive;
 
-/** @brief The @c min-backups directive */
-static const struct MinBackupsDirective: InheritableDirective {
-  MinBackupsDirective(): InheritableDirective("min-backups", 1, 1) {}
+/** @brief The @c backup-policy directive */
+static const struct BackupPolicyDirective: InheritableDirective {
+  BackupPolicyDirective(): InheritableDirective("backup-policy", 1, 1) {}
   void set(ConfContext &cc) const override {
-    warning(WARNING_DEPRECATED,
-            "%s:%d: the 'min-backups' directive is deprecated, use 'prune-parameter min-backups' instead",
-            cc.path.c_str(), cc.line);
-    parseInteger(cc.bits[1], 1);
-    cc.context->pruneParameters["min-backups"] = cc.bits[1];
+    cc.context->backupPolicy = cc.bits[1];
   }
-} min_backups_directive;
+} backup_policy_directive;
 
-/** @brief The @c prune-age directive */
-static const struct PruneAgeDirective: InheritableDirective {
-  PruneAgeDirective(): InheritableDirective("prune-age", 1, 1) {}
-  void set(ConfContext &cc) const override {
-    warning(WARNING_DEPRECATED,
-            "%s:%d: the 'prune-age' directive is deprecated, use 'prune-parameter prune-age' instead",
-            cc.path.c_str(), cc.line);
-    parseInteger(cc.bits[1], 1);
-    cc.context->pruneParameters["prune-age"] = cc.bits[1];
+/** @brief The @c backup-parameter directive */
+static const struct BackupParameterDirective: InheritableDirective {
+  BackupParameterDirective(): InheritableDirective("backup-parameter", 2, 2) {}
+  void check(const ConfContext &cc) const override {
+    ConfDirective::check(cc);
+    const std::string &name = (cc.bits[1] != "--remove" ?
+                                             cc.bits[1] : cc.bits[2]);
+    if(!valid(name))
+      throw SyntaxError("invalid backup-parameter name");
   }
-} prune_age_directive;
+  void set(ConfContext &cc) const override {
+    if(cc.bits[1] != "--remove")
+      cc.context->backupParameters[cc.bits[1]] = cc.bits[2];
+    else
+      cc.context->backupParameters.erase(cc.bits[2]);
+  }
+
+  /** @brief Test for valid @c backup-parameter names
+   * @param name Candidate name
+   * @return @c true if @name is valid
+   */
+  static bool valid(const std::string &name) {
+    return name.size() > 0
+      && name.at(0) != '-'
+      && name.find_first_not_of(POLICY_PARAMETER_VALID) == std::string::npos;
+  }
+} backup_parameter_directive;
 
 /** @brief The @c prune-policy directive */
 static const struct PrunePolicyDirective: InheritableDirective {
@@ -547,7 +551,7 @@ static const struct PruneParameterDirective: InheritableDirective {
   static bool valid(const std::string &name) {
     return name.size() > 0
       && name.at(0) != '-'
-      && name.find_first_not_of(PRUNE_PARAMETER_VALID) == std::string::npos;
+      && name.find_first_not_of(POLICY_PARAMETER_VALID) == std::string::npos;
   }
 } prune_parameter_directive;
 
@@ -571,7 +575,8 @@ static const struct PostBackupHookDirective: InheritableDirective {
 static const struct RsyncTimeoutDirective: InheritableDirective {
   RsyncTimeoutDirective(): InheritableDirective("rsync-timeout", 1, 1) {}
   void set(ConfContext &cc) const override {
-    cc.context->rsyncTimeout = parseInteger(cc.bits[1], 1);
+    cc.context->rsyncTimeout = parseInteger(cc.bits[1], 1,
+                                            std::numeric_limits<int>::max());
   }
 } rsync_timeout_directive;
 
@@ -579,7 +584,8 @@ static const struct RsyncTimeoutDirective: InheritableDirective {
 static const struct HookTimeoutDirective: InheritableDirective {
   HookTimeoutDirective(): InheritableDirective("hook-timeout", 1, 1) {}
   void set(ConfContext &cc) const override {
-    cc.context->hookTimeout = parseInteger(cc.bits[1], 1);
+    cc.context->hookTimeout = parseInteger(cc.bits[1], 1,
+                                           std::numeric_limits<int>::max());
   }
 } hook_timeout_directive;
 
@@ -604,7 +610,8 @@ static const struct HostCheckDirective: InheritableDirective {
 static const struct SshTimeoutDirective: InheritableDirective {
   SshTimeoutDirective(): InheritableDirective("ssh-timeout", 1, 1) {}
   void set(ConfContext &cc) const override {
-    cc.context->sshTimeout = parseInteger(cc.bits[1], 1);
+    cc.context->sshTimeout = parseInteger(cc.bits[1], 1,
+                                          std::numeric_limits<int>::max());
   }
 } ssh_timeout_directive;
 
@@ -648,6 +655,14 @@ static const struct HostDirective: public ConfDirective {
   }
 } host_directive;
 
+/** @brief The @c group directive */
+static const struct GroupDirective: public HostOnlyDirective {
+  GroupDirective(): HostOnlyDirective("group", 1, 1) {}
+  void set(ConfContext &cc) const override {
+    cc.host->group = cc.bits[1];
+  }
+} group_directive;
+
 /** @brief The @c hostname directive */
 static const struct HostnameDirective: public HostOnlyDirective {
   HostnameDirective(): HostOnlyDirective("hostname", 1, 1) {}
@@ -658,7 +673,7 @@ static const struct HostnameDirective: public HostOnlyDirective {
 
 /** @brief The @c always-up directive */
 static const struct AlwaysUpDirective: public HostOnlyDirective {
-  AlwaysUpDirective(): HostOnlyDirective("always-up", 0, 1) {}
+  AlwaysUpDirective(): HostOnlyDirective("always-up", 1, 1) {}
   void set(ConfContext &cc) const override {
     warning(WARNING_DEPRECATED,
             "%s:%d: the 'always-up' directive is deprecated, use 'host-check always-up' instead",
@@ -671,7 +686,9 @@ static const struct AlwaysUpDirective: public HostOnlyDirective {
 static const struct PriorityDirective: public HostOnlyDirective {
   PriorityDirective(): HostOnlyDirective("priority", 1, 1) {}
   void set(ConfContext &cc) const override {
-    cc.host->priority = parseInteger(cc.bits[1]);
+    cc.host->priority = parseInteger(cc.bits[1],
+                                     std::numeric_limits<int>::min(),
+                                     std::numeric_limits<int>::max());
   }
 } priority_directive;
 
@@ -733,7 +750,7 @@ static const struct CheckFileDirective: public VolumeOnlyDirective {
 
 /** @brief The @c check-mounted directive */
 static const struct CheckMountedDirective: public VolumeOnlyDirective {
-  CheckMountedDirective(): VolumeOnlyDirective("check-mounted", 0, 1) {}
+  CheckMountedDirective(): VolumeOnlyDirective("check-mounted", 1, 1) {}
   void set(ConfContext &cc) const override {
     cc.volume->checkMounted = get_boolean(cc);
   }
