@@ -92,6 +92,12 @@ void Host::write(std::ostream &os, int step, bool verbose) const {
   ConfBase::write(os, step, verbose);
   d(os, "", step);
 
+  d(os, "# Concurrency group", step);
+  d(os, "#   group NAME", step);
+  if(group != name)
+    os << indent(step) << "group " << quote(group) << '\n';
+  d(os, "", step);
+
   d(os, "# Hostname for SSH", step);
   d(os, "#   hostname NAME", step);
   os << indent(step) << "hostname " << quote(hostname) << '\n';
@@ -103,10 +109,12 @@ void Host::write(std::ostream &os, int step, bool verbose) const {
     os << indent(step) << "user " << quote(user) << '\n';
   d(os, "", step);
 
-  d(os, "# Treat host being down as an error", step);
-  d(os, "#   always-up true|false", step);
-  os << indent(step) << "always-up " << (alwaysUp ? "true" : "false") << '\n';
-  d(os, "", step);
+  if(alwaysUp) {
+    d(os, "# Treat host being down as an error", step);
+    d(os, "#   always-up true|false", step);
+    os << indent(step) << "always-up " << (alwaysUp ? "true" : "false") << '\n';
+    d(os, "", step);
+  }
 
   d(os, "# Glob pattern for devices this host will be backed up to", step);
   d(os, "#   devices PATTERN", step);
