@@ -1,4 +1,4 @@
-// Copyright © 2011, 2012, 2014-2018 Richard Kettlewell.
+// Copyright © 2011, 2012, 2014-2019 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include <boost/range/adaptor/reversed.hpp>
 #include <boost/filesystem.hpp>
 #include <sysexits.h>
+#include <thread>
 
 /** @brief rsync exit status indicating a file vanished during backup */
 const int RERR_VANISHED = 24;
@@ -119,10 +120,10 @@ public:
    */
   const Backup *getLastBackup();
 
-  /** @brief Set up the common hook environment for a subprocess
+  /** @brief Set up the common environment for a subprocess
    * @param sp Subprocess
    */
-  void hookEnvironment(Subprocess &sp);
+  void setEnvironment(Subprocess &sp);
 
   /** @brief Set up logfile IO for a subprocess
    * @param sp Subprocess
@@ -145,6 +146,19 @@ public:
 
   /** @brief Perform a backup */
   void performBackup();
+
+  /** @brief Return the ID for a new backup */
+  static std::string backupID() {
+    time_t now = Date::now();           // overridden by ${RSBACKUP_TIME}
+    struct tm t;
+    if(!gmtime_r(&now, &t))
+      throw SystemError("gmtime_r", errno);
+    char buffer[64];
+    snprintf(buffer, sizeof buffer, "%04d-%02d-%02dT%02d:%02d:%02d",
+             t.tm_year + 1900, t.tm_mon + 1, t.tm_mday,
+             t.tm_hour, t.tm_min, t.tm_sec);
+    return buffer;
+  }
 };
 
 MakeBackup::MakeBackup(Volume *volume_, Device *device_):
@@ -153,7 +167,7 @@ MakeBackup::MakeBackup(Volume *volume_, Device *device_):
   host(volume->parent),
   startTime(Date::now()),
   today(Date::today()),
-  id(today.toString()),
+  id(backupID()),
   volumePath(device->store->path
              + PATH_SEP + host->name
              + PATH_SEP + volume->name),
@@ -181,17 +195,17 @@ const Backup *MakeBackup::getLastBackup() {
   return nullptr;
 }
 
-void MakeBackup::hookEnvironment(Subprocess &sp) {
+void MakeBackup::setEnvironment(Subprocess &sp) {
   sp.setenv("RSBACKUP_DEVICE", device->name);
   sp.setenv("RSBACKUP_HOST", host->name);
+  sp.setenv("RSBACKUP_GROUP", host->group);
   sp.setenv("RSBACKUP_SSH_HOSTNAME", host->hostname);
   sp.setenv("RSBACKUP_SSH_USERNAME", host->user);
   sp.setenv("RSBACKUP_SSH_TARGET", host->userAndHost());
   sp.setenv("RSBACKUP_STORE", device->store->path);
   sp.setenv("RSBACKUP_VOLUME", volume->name);
   sp.setenv("RSBACKUP_VOLUME_PATH", volume->path);
-  sp.setenv("RSBACKUP_ACT", command.act ? "true" : "false");
-  sp.setTimeout(volume->hookTimeout);
+  sp.setenv("RSBACKUP_ACT", globalCommand.act ? "true" : "false");
 }
 
 void MakeBackup::subprocessIO(Subprocess &sp, bool outputToo) {
@@ -211,8 +225,9 @@ int MakeBackup::preBackup() {
                   volume->preBackup);
     sp.capture(1, &output);
     sp.setenv("RSBACKUP_HOOK", "pre-backup-hook");
-    hookEnvironment(sp);
-    sp.reporting(warning_mask & WARNING_VERBOSE, false);
+    setEnvironment(sp);
+    sp.setTimeout(volume->hookTimeout);
+    sp.reporting(globalWarningMask & WARNING_VERBOSE, false);
     subprocessIO(sp, false);
     al.add(&sp);
     al.go();
@@ -284,8 +299,6 @@ int MakeBackup::rsyncBackup() {
     cmd.push_back(host->rsyncCommand);
     cmd.insert(cmd.end(), volume->rsyncBaseOptions.begin(), volume->rsyncBaseOptions.end());
     cmd.insert(cmd.end(), volume->rsyncExtraOptions.begin(), volume->rsyncExtraOptions.end());
-    if(!(warning_mask & WARNING_VERBOSE))
-      cmd.push_back("--quiet");         // suppress non-errors
     if(!volume->traverse)
       cmd.push_back("--one-file-system"); // don't cross mount points
     // Exclusions
@@ -304,24 +317,19 @@ int MakeBackup::rsyncBackup() {
                        + volume->name + "/"
                        + device->name);
     sp.setCommand(cmd);
-    sp.reporting(warning_mask & WARNING_VERBOSE, !command.act);
+    setEnvironment(sp);
+    sp.reporting(globalWarningMask & WARNING_VERBOSE, !globalCommand.act);
     sp.after(before_backup.get_name(), ACTION_SUCCEEDED);
-    // Clean up when finished
-    // TODO should just unlink, rm -rf is overkill.
-    BulkRemove after_backup("after-backup/"
-                            + volume->parent->name + "/"
-                            + volume->name + "/"
-                            + device->name,
-                            incompletePath);
-    al.add(&after_backup);
-    after_backup.after(sp.get_name(), ACTION_SUCCEEDED);
-    if(!command.act)
+    if(!globalCommand.act)
       return 0;
     subprocessIO(sp, true);
     sp.setTimeout(volume->rsyncTimeout);
-    // Make the backup
+    // Make the backup, with the global lock released
     al.add(&sp);
-    al.go();
+    {
+      release_guard<std::mutex> globalRelease(globalLock);
+      al.go();
+    }
     rc = sp.getStatus();
     what = "rsync";
     // Suppress exit status 24 "Partial transfer due to vanished source files"
@@ -332,6 +340,12 @@ int MakeBackup::rsyncBackup() {
               device->name.c_str());
       rc = 0;
     }
+    // Clean up when finished
+    if(rc == 0) {
+      if(remove(incompletePath.c_str()) < 0) {
+        throw SystemError("removing " + incompletePath, errno);
+      }
+    }
   } catch(std::runtime_error &e) {
     // Try to handle any other errors the same way as rsync failures.  If we
     // can't even write to the logfile we error out.
@@ -357,8 +371,9 @@ void MakeBackup::postBackup() {
                   volume->postBackup);
     sp.setenv("RSBACKUP_STATUS", outcome && outcome->rc == 0 ? "ok" : "failed");
     sp.setenv("RSBACKUP_HOOK", "post-backup-hook");
-    hookEnvironment(sp);
-    sp.reporting(warning_mask & WARNING_VERBOSE, false);
+    setEnvironment(sp);
+    sp.setTimeout(volume->hookTimeout);
+    sp.reporting(globalWarningMask & WARNING_VERBOSE, false);
     subprocessIO(sp, true);
     al.add(&sp);
     al.go();
@@ -370,7 +385,7 @@ void MakeBackup::performBackup() {
   what = "preBackup";
   int rc = preBackup();
   if(WIFEXITED(rc) && WEXITSTATUS(rc) == EX_TEMPFAIL) {
-    if(warning_mask & WARNING_VERBOSE)
+    if(globalWarningMask & WARNING_VERBOSE)
       IO::out.writef("INFO: %s:%s is temporarily unavailable due to pre-backup-hook\n",
                      host->name.c_str(),
                      volume->name.c_str());
@@ -382,21 +397,20 @@ void MakeBackup::performBackup() {
   outcome = new Backup();
   outcome->rc = rc;
   outcome->time = startTime;
-  outcome->date = today;
   outcome->id = id;
   outcome->deviceName = device->name;
   outcome->volume = volume;
   outcome->setStatus(UNDERWAY);
-  if(command.act) {
+  if(globalCommand.act) {
     // Record in the database that the backup is underway
     // If this fails then the backup just fails.
-    config.getdb().begin();
-    outcome->insert(config.getdb(), true/*replace*/);
-    config.getdb().commit();
+    globalConfig.getdb().begin();
+    outcome->insert(globalConfig.getdb(), true/*replace*/);
+    globalConfig.getdb().commit();
   }
   // Run the post-backup hook
   postBackup();
-  if(!command.act) {
+  if(!globalCommand.act) {
     delete outcome;
     return;
   }
@@ -409,8 +423,8 @@ void MakeBackup::performBackup() {
   volume->addBackup(outcome);
   if(rc) {
     // Count up errors
-    ++errors;
-    if(warning_mask & (WARNING_VERBOSE|WARNING_ERRORLOGS)) {
+    ++globalErrors;
+    if(globalWarningMask & (WARNING_VERBOSE|WARNING_ERRORLOGS)) {
       warning(WARNING_VERBOSE|WARNING_ERRORLOGS,
               "backup of %s:%s to %s: %s",
               host->name.c_str(),
@@ -431,9 +445,9 @@ void MakeBackup::performBackup() {
   for(;;) {
     int retries = 0;
     try {
-      config.getdb().begin();
-      outcome->update(config.getdb());
-      config.getdb().commit();
+      globalConfig.getdb().begin();
+      outcome->update(globalConfig.getdb());
+      globalConfig.getdb().commit();
       break;
     } catch(DatabaseBusy &) {
       // Log a message every second or so
@@ -455,7 +469,7 @@ void MakeBackup::performBackup() {
 // device->store is assumed to be set.
 static void backupVolume(Volume *volume, Device *device) {
   Host *host = volume->parent;
-  if(warning_mask & WARNING_VERBOSE)
+  if(globalWarningMask & WARNING_VERBOSE)
     IO::out.writef("INFO: backup %s:%s to %s\n",
                    host->name.c_str(), volume->name.c_str(),
                    device->name.c_str());
@@ -463,76 +477,106 @@ static void backupVolume(Volume *volume, Device *device) {
   mb.performBackup();
 }
 
-// Backup VOLUME
-static void backupVolume(Volume *volume) {
+// Backup VOLUME onto DEVICE, if possible.
+//
+// The device lock is held.
+static void maybeBackupVolume(Volume *volume, Device *device) {
   Host *host = volume->parent;
   char buffer[1024];
-  for(auto &d: config.devices) {
-    Device *device = d.second;
-    switch(volume->needsBackup(device)) {
-    case BackupRequired:
-      config.identifyDevices(Store::Enabled);
-      if(device->store && device->store->state == Store::Enabled)
-        backupVolume(volume, device);
-      else if(warning_mask & WARNING_STORE) {
-        config.identifyDevices(Store::Disabled);
-        if(device->store)
-          switch(device->store->state) {
-          case Store::Disabled:
-            warning(WARNING_STORE,
-                    "cannot backup %s:%s to %s - device suppressed due to --store",
-                    host->name.c_str(),
-                    volume->name.c_str(),
-                    device->name.c_str());
-            break;
-          default:
-            snprintf(buffer, sizeof buffer,
-                     "device %s store %s unexpected state %d",
-                     device->name.c_str(),
-                     device->store->path.c_str(),
-                     device->store->state);
-            throw FatalStoreError(buffer);
-          }
-        else
+  switch(volume->needsBackup(device)) {
+  case BackupRequired:
+    globalConfig.identifyDevices(Store::Enabled);
+    if(device->store && device->store->state == Store::Enabled)
+      backupVolume(volume, device);
+    else if(globalWarningMask & WARNING_STORE) {
+      globalConfig.identifyDevices(Store::Disabled);
+      if(device->store)
+        switch(device->store->state) {
+        case Store::Disabled:
           warning(WARNING_STORE,
-                  "cannot backup %s:%s to %s - device not available",
+                  "cannot backup %s:%s to %s - device suppressed due to --store",
                   host->name.c_str(),
                   volume->name.c_str(),
                   device->name.c_str());
+          break;
+        default:
+          snprintf(buffer, sizeof buffer,
+                   "device %s store %s unexpected state %d",
+                   device->name.c_str(),
+                   device->store->path.c_str(),
+                   device->store->state);
+          throw FatalStoreError(buffer);
+        }
+      else
+        warning(WARNING_STORE,
+                "cannot backup %s:%s to %s - device not available",
+                host->name.c_str(),
+                volume->name.c_str(),
+                device->name.c_str());
+    }
+    break;
+  case AlreadyBackedUp:
+    if(globalWarningMask & WARNING_VERBOSE)
+      IO::out.writef("INFO: %s:%s is already backed up on %s\n",
+                     host->name.c_str(),
+                     volume->name.c_str(),
+                     device->name.c_str());
+    break;
+  case NotAvailable:
+    if(globalWarningMask & WARNING_VERBOSE)
+      IO::out.writef("INFO: %s:%s is not available\n",
+                     host->name.c_str(),
+                     volume->name.c_str());
+    break;
+  case NotThisDevice:
+    if(globalWarningMask & WARNING_VERBOSE)
+      IO::out.writef("INFO: %s:%s excluded from %s by device pattern\n",
+                     host->name.c_str(),
+                     volume->name.c_str(),
+                     device->name.c_str());
+    break;
+  }
+}
+
+// Backup VOLUME
+static void backupVolume(Volume *volume) {
+  // Build a list of devices
+  std::set<Device *> devices;
+  for(auto &d: globalConfig.devices) {
+    devices.insert(d.second);
+  }
+  while(devices.size() > 0) {
+    bool worked = false;
+    // Look for a device we can lock
+    for(auto device: devices) {
+      if(device->lock.try_lock()) {
+        std::lock_guard<std::mutex> guard(device->lock, std::adopt_lock);
+        maybeBackupVolume(volume, device);
+        devices.erase(device);
+        worked = true;
+        break;
       }
-      break;
-    case AlreadyBackedUp:
-      if(warning_mask & WARNING_VERBOSE)
-        IO::out.writef("INFO: %s:%s is already backed up on %s\n",
-                       host->name.c_str(),
-                       volume->name.c_str(),
-                       device->name.c_str());
-      break;
-    case NotAvailable:
-      if(warning_mask & WARNING_VERBOSE)
-        IO::out.writef("INFO: %s:%s is not available\n",
-                       host->name.c_str(),
-                       volume->name.c_str());
-      break;
-    case NotThisDevice:
-      if(warning_mask & WARNING_VERBOSE)
-        IO::out.writef("INFO: %s:%s excluded from %s by device pattern\n",
-                       host->name.c_str(),
-                       volume->name.c_str(),
-                       device->name.c_str());
-      break;
+    }
+    // If we didn't find a suitable volume wait a bit and try again
+    if(!worked) {
+      release_guard<std::mutex> globalRelease(globalLock);
+      usleep(100*000/*µs*/);
     }
   }
 }
 
 // Backup HOST
-static void backupHost(Host *host) {
+static void backupHost(Host *host, std::mutex *lock) {
   // Do a quick check for unavailable hosts
-  if(!host->available()) {
+  bool available = host->available();
+  // Serialize host groups
+  std::lock_guard<std::mutex> groupGuard(*lock);
+  std::lock_guard<std::mutex> globalGuard(globalLock);
+  if(!available) {
     if(host->alwaysUp) {
       warning(WARNING_ALWAYS, "cannot backup always-up host %s - not reachable",
               host->name.c_str());
-      ++errors;
+      ++globalErrors;
       // Try anyway, so that the failures are recorded.
     } else {
       warning(WARNING_UNREACHABLE, "cannot backup %s - not reachable", host->name.c_str());
@@ -557,14 +601,38 @@ static bool order_host(const Host *a, const Host *b) {
 // Backup everything
 void makeBackups() {
   // Load up log files
-  config.readState();
+  globalConfig.readState();
   std::vector<Host *> hosts;
-  for(auto &h: config.hosts) {
+  for(auto &h: globalConfig.hosts) {
     Host *host = h.second;
     if(host->selected())
       hosts.push_back(host);
   }
   std::sort(hosts.begin(), hosts.end(), order_host);
-  for(Host *h: hosts)
-    backupHost(h);
+  // Create concurrency group locks
+  std::map<std::string,std::mutex *> locks;
+  for(Host *h: hosts) {
+    if(locks.find(h->group) == locks.end())
+      locks[h->group] = new std::mutex();
+  }
+  // Initiate backups in threads
+  std::map<std::string,std::thread *> threads;
+  for(Host *h: hosts) {
+    threads[h->name] = new std::thread(backupHost, h, locks[h->group]);
+  }
+  {
+    // Release the global lock while we wait for the threads
+    release_guard<std::mutex> globalRelease(globalLock);
+    // Wait for the threads
+    for(auto it: threads) {
+      it.second->join();
+    }
+  }
+  // Clean up the locks and threads
+  for(auto it: threads) {
+    delete it.second;
+  }
+  for(auto it: locks) {
+    delete it.second;
+  }
 }
