@@ -21,6 +21,7 @@
 #include "Subprocess.h"
 #include "Utils.h"
 #include "Store.h"
+#include "BackupPolicy.h"
 #include <cstdio>
 #include <ostream>
 #include <fnmatch.h>
@@ -58,18 +59,20 @@ void Volume::calculate() {
     if(backup->getStatus() == COMPLETE) {
       // Global figures
       ++completed;
-      if(completed == 1 || backup->date < oldest)
-        oldest = backup->date;
-      if(completed == 1 || backup->date > newest)
-        newest = backup->date;
+      if(completed == 1 || backup->time < oldest)
+        oldest = backup->time;
+      if(completed == 1 || backup->time > newest)
+        newest = backup->time;
 
       // Per-device figures
       Volume::PerDevice &pd = perDevice[backup->deviceName];
       ++pd.count;
-      if(pd.count == 1 || backup->date < pd.oldest)
-        pd.oldest = backup->date;
-      if(pd.count == 1 || backup->date > pd.newest)
-        pd.newest = backup->date;
+      if(pd.count == 1 || backup->time < pd.oldest)
+        pd.oldest = backup->time;
+      if(pd.count == 1 || backup->time > pd.newest) {
+        pd.newest = backup->time;
+        pd.size = backup->getSize();
+      }
     }
   }
   for(auto it = perDevice.begin(); it != perDevice.end();) {
@@ -184,12 +187,9 @@ BackupRequirement Volume::needsBackup(Device *device) {
     /* fail safe - make the backup */
     break;
   }
-  Date today = Date::today();
-  for(const Backup *backup: backups)
-    if(backup->rc == 0
-       && backup->date == today
-       && backup->deviceName == device->name)
-      return AlreadyBackedUp;           // Already backed up
+  const BackupPolicy *policy = BackupPolicy::find(backupPolicy);
+  if(!policy->backup(this, device))
+    return AlreadyBackedUp;
   if(!available())
     return NotAvailable;
   return BackupRequired;
